"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[6450],{506450:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M19.89 10.55A1 1 0 0 0 19 10h-5V4a1 1 0 0 0-.69-1 1 1 0 0 0-1.12.36l-8 11a1 1 0 0 0-.08 1A1 1 0 0 0 5 16h5v6a1 1 0 0 0 1 1 1 1 0 0 0 .81-.41l8-11a1 1 0 0 0 .08-1.04M12 18.92V15a1 1 0 0 0-1-1H7l5-6.92V11a1 1 0 0 0 1 1h4z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=4ac14c9926de83775c76.chunk.js.map