"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2426],{572426:(e,l,t)=>{t.r(l),t.d(l,{default:()=>s});var n,r,a=t(667294);function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},i.apply(null,arguments)}const s=e=>a.createElement("svg",i({xmlns:"http://www.w3.org/2000/svg",width:24,height:25,fill:"none"},e),n||(n=a.createElement("rect",{width:16,height:16,x:4,y:4.5,fill:"currentColor",rx:2})),r||(r=a.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"m14.72 9.29-4.29 4.3-1.65-1.65a1 1 0 1 0-1.41 1.41l2.35 2.36a1 1 0 0 0 1.41 0l5-5a1 1 0 1 0-1.41-1.42",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=6ec4db8a4f871b54884d.chunk.js.map