# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Increase size of name column in ab_view_menu

Revision ID: cefabc8f7d38
Revises: 6c7537a6004a
Create Date: 2018-12-13 15:38:36.772750

"""

# revision identifiers, used by Alembic.
revision = "cefabc8f7d38"
down_revision = "6c7537a6004a"

import sqlalchemy as sa  # noqa: E402
from alembic import op  # noqa: E402


def upgrade():
    with op.batch_alter_table("ab_view_menu") as batch_op:
        batch_op.alter_column(
            "name",
            existing_type=sa.String(length=100),
            existing_nullable=False,
            type_=sa.String(length=255),
            nullable=False,
        )


def downgrade():
    with op.batch_alter_table("ab_view_menu") as batch_op:
        batch_op.alter_column(
            "name",
            existing_type=sa.String(length=255),
            existing_nullable=False,
            type_=sa.String(length=100),
            nullable=False,
        )
