"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[4002],{814002:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a,r=n(667294);function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},i.apply(null,arguments)}const s=e=>r.createElement("svg",i({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m17.346 14.746-4.025-7a1.5 1.5 0 0 0-2.62 0l-4 7a1.5 1.5 0 0 0 1.28 2.265h8.06a1.5 1.5 0 0 0 1.305-2.265",clipRule:"evenodd"})),a||(a=r.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"M11.51 10.51a.5.5 0 0 1 1 0v2a.5.5 0 0 1-1 0zm0 4a.5.5 0 1 1 1 0 .5.5 0 0 1-1 0",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=9e0f9a0dc7d4f836e7b9.chunk.js.map