"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[9480],{489480:(e,l,n)=>{n.r(l),n.d(l,{default:()=>h});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const h=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M21 2H3a1 1 0 0 0-1 1v18a1 1 0 0 0 1 1h18a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1M11 20H4v-7h7zm0-9H4V4h7zm9 9h-7v-7h7zm0-9h-7V4h7z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=46aa737794f86e4fcc88.chunk.js.map