"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[3211],{633211:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M12 2C6.477 2 2 6.477 2 12s4.477 10 10 10 10-4.477 10-10A10 10 0 0 0 12 2m0 18a8 8 0 1 1 0-16 8 8 0 0 1 0 16m0-14a1 1 0 0 0-1 1v4.42l-2.1 1.21a1 1 0 0 0 .5 1.87 1 1 0 0 0 .5-.13l2.6-1.5.09-.09.16-.13a1 1 0 0 0 .1-.16 1 1 0 0 0 .08-.17.7.7 0 0 0 .05-.2L13 12V7a1 1 0 0 0-1-1",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=e00bfb29c9cca35617c2.chunk.js.map