"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[6362],{690170:(s,e,r)=>{r.r(e),r.d(e,{default:()=>l});var u=r(616550),a=r(257001),t=r(135944);const l=()=>{const{idOrSlug:s}=(0,u.UO)();return(0,t.tZ)(a.DashboardPage,{idOrSlug:s})}}}]);
//# sourceMappingURL=4e6e87bc9b6afbc046a0.chunk.js.map