"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7618],{327618:(e,a,l)=>{l.r(a),l.d(a,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var l=arguments[a];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M4 5a1 1 0 1 0 0-2 1 1 0 0 0 0 2m0 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2m5-5a1 1 0 1 1-2 0 1 1 0 0 1 2 0m3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2m5-1a1 1 0 1 1-2 0 1 1 0 0 1 2 0m3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2m1 3a1 1 0 1 1-2 0 1 1 0 0 1 2 0M4 11h16a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1v-8a1 1 0 0 1 1-1m1 2v6h14v-6z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=72f4300c6f16f17dcb9f.chunk.js.map