"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2236],{611950:(e,t,n)=>{var r=n(564836),o=n(475263);Object.defineProperty(t,"__esModule",{value:!0}),t.default=t.HOOK_MARK=void 0;var a=o(n(667294)),i=r(n(645520));t.HOOK_MARK="RC_FORM_INTERNAL_HOOKS";var l=function(){(0,i.default)(!1,"Can not find FormContext. Please make sure you wrap Field under Form.")},u=a.createContext({getFieldValue:l,getFieldsValue:l,getFieldError:l,getFieldsError:l,isFieldsTouched:l,isFieldTouched:l,isFieldValidating:l,isFieldsValidating:l,resetFields:l,setFields:l,setFieldsValue:l,validateFields:l,submit:l,getInternalHooks:function(){return l(),{dispatch:l,initEntityValue:l,registerField:l,useSubscribe:l,setInitialValues:l,setCallbacks:l,getFields:l,setValidateMessages:l,setPreserve:l}}});t.default=u},367265:(e,t,n)=>{var r=n(475263).default;Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r=o.useRef({});return"value"in r.current&&!n(r.current.condition,t)||(r.current.value=e(),r.current.condition=t),r.current.value};var o=r(n(667294))},564543:(e,t)=>{Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var n=function(e){return+setTimeout(e,16)},r=function(e){return clearTimeout(e)};"undefined"!=typeof window&&"requestAnimationFrame"in window&&(n=function(e){return window.requestAnimationFrame(e)},r=function(e){return window.cancelAnimationFrame(e)});var o=0,a=new Map;function i(e){a.delete(e)}var l=function(e){var t=o+=1;return function r(o){if(0===o)i(t),e();else{var l=n((function(){r(o-1)}));a.set(t,l)}}(arguments.length>1&&void 0!==arguments[1]?arguments[1]:1),t};l.cancel=function(e){var t=a.get(e);return i(e),r(t)},t.default=l},475531:(e,t,n)=>{var r=n(564836).default;Object.defineProperty(t,"__esModule",{value:!0}),t.useComposeRef=t.supportRef=t.supportNodeRef=t.getNodeRef=t.fillRef=t.composeRef=void 0;var o=r(n(918698)),a=n(667294),i=n(211805),l=r(n(367265)),u=t.fillRef=function(e,t){"function"==typeof e?e(t):"object"===(0,o.default)(e)&&e&&"current"in e&&(e.current=t)},s=t.composeRef=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var r=t.filter(Boolean);return r.length<=1?r[0]:function(e){t.forEach((function(t){u(t,e)}))}},f=(t.useComposeRef=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return(0,l.default)((function(){return s.apply(void 0,t)}),t,(function(e,t){return e.length!==t.length||e.every((function(e,n){return e!==t[n]}))}))},t.supportRef=function(e){var t,n,r=(0,i.isMemo)(e)?e.type.type:e.type;return!!("function"!=typeof r||null!==(t=r.prototype)&&void 0!==t&&t.render||r.$$typeof===i.ForwardRef)&&!!("function"!=typeof e||null!==(n=e.prototype)&&void 0!==n&&n.render||e.$$typeof===i.ForwardRef)});function c(e){return(0,a.isValidElement)(e)&&!(0,i.isFragment)(e)}t.supportNodeRef=function(e){return c(e)&&f(e)},t.getNodeRef=Number(a.version.split(".")[0])>=19?function(e){return c(e)?e.props.ref:null}:function(e){return c(e)?e.ref:null}},47249:(e,t,n)=>{n.d(t,{Z:()=>i});var r=n(990317),o=n(769842),a=n(269244);function i(e,t=!1){const n=t?r.i$:r.g0,i=n("%Y-%m-%d %H:%M:%S.%L"),l=n("%Y-%m-%d %H:%M:%S"),u=n("%Y-%m-%d %H:%M"),s=n("%Y-%m-%d %H:%M"),f=n("%Y-%m-%d"),c=n("%Y-%m-%d"),d=n("%Y"),{hasMillisecond:p,hasSecond:m,hasMinute:v,hasHour:h,isNotFirstDayOfMonth:g,isNotFirstMonth:y}=t?o.Z:o.b;let S=d;return e.forEach((e=>{S===d&&y(e)&&(S=c),S===c&&g(e)&&(S=f),S===f&&h(e)&&(S=s),S===s&&v(e)&&(S=u),S===u&&m(e)&&(S=l),S===l&&p(e)&&(S=i)})),new a.Z({description:"Use the finest grain in an array of dates to format all dates in the array",formatFunc:S,id:"finest_temporal_grain",label:"Format temporal columns with the finest grain",useLocalTime:t})}},804591:(e,t,n)=>{n.d(t,{Z:()=>o});var r=n(897538);const o=(0,n(751995).iK)(r.Z.Item)`
  ${({theme:e})=>`\n    .ant-form-item-label {\n      padding-bottom: ${e.gridUnit}px;\n      & > label {\n        text-transform: uppercase;\n        font-size: ${e.typography.sizes.s}px;\n        color: ${e.colors.grayscale.base};\n\n        &.ant-form-item-required:not(.ant-form-item-required-mark-optional) {\n          &::before {\n            display: none;\n          }\n          &::after {\n            display: inline-block;\n            color: ${e.colors.error.base};\n            font-size: ${e.typography.sizes.s}px;\n            content: '*';\n          }\n        }\n      }\n    }\n  `}
`},861641:(e,t,n)=>{var r,o;n.d(t,{N:()=>o,p:()=>r}),function(e){e.Simple="SIMPLE",e.Sql="SQL"}(r||(r={})),function(e){e.Having="HAVING",e.Where="WHERE"}(o||(o={}))},345181:(e,t,n)=>{n.r(t),n.d(t,{default:()=>F});var r=n(23279),o=n.n(r),a=n(667294),i=n(355786),l=n(310581),u=n(47249),s=n(61988),f=n(616355),c=n(890057),d=n(104715),p=n(427600),m=n(985633),v=n(52004),h=n(174448),g=n(356080),y=n(135944);function S(e,t){switch(t.type){case"ownState":return e.ownState={...e.ownState,...t.ownState},e;case"filterState":return JSON.stringify(e.extraFormData)!==JSON.stringify(t.extraFormData)&&(e.extraFormData=t.extraFormData),JSON.stringify(e.filterState)!==JSON.stringify(t.filterState)&&(e.filterState={...e.filterState,...t.filterState}),e;default:return e}}function F(e){const{coltypeMap:t,data:n,filterState:r,formData:F,height:b,isRefreshing:w,width:M,setDataMask:N,setHoveredFilter:O,unsetHoveredFilter:C,setFocusedFilter:R,unsetFocusedFilter:x,setFilterActive:$,appSection:E,showOverflow:Z,parentRef:A,inputRef:T,filterBarOrientation:_}=e,{enableEmptyFilter:k,multiSelect:H,showSearch:V,inverseSelection:j,defaultToFirstItem:D,searchAllOptions:I}=F,K=(0,a.useMemo)((()=>(0,i.Z)(F.groupby).map(l.Z)),[F.groupby]),[P]=K,[q]=(0,a.useState)(t),[J,L]=(0,a.useState)(""),[Y,z]=(0,c.C)(S,{extraFormData:{},filterState:r}),B=t[P],G=(0,a.useMemo)((()=>(0,g.EG)({timeFormatter:(0,u.Z)(n.map((e=>e[P])))})),[n,P]),U=(0,a.useCallback)((e=>{const t=k&&!j&&!(null!=e&&e.length),n=j&&null!=e&&e.length?(0,s.t)(" (excluded)"):"";z({type:"filterState",extraFormData:(0,g.vb)(P,e,t,j),filterState:{...r,label:null!=e&&e.length?`${(e||[]).map((e=>G(e,B))).join(", ")}${n}`:void 0,value:E===f.Tr.FilterConfigModal&&D?void 0:e}})}),[E,P,B,D,z,k,j,JSON.stringify(r),G]),W=E===f.Tr.FilterConfigModal&&D,Q=(0,a.useMemo)((()=>o()((e=>{L(e),I&&z({type:"ownState",ownState:{coltypeMap:q,search:e}})}),p.M$)),[z,q,I]),X=(0,a.useCallback)((()=>{x(),Q("")}),[Q,x]),ee=(0,a.useCallback)((e=>{const t=null===e?[null]:(0,i.Z)(e);0===t.length?U(null):U(t)}),[U]),te=0===n.length?(0,s.t)("No data"):(0,s.tn)("%s option","%s options",n.length,n.length),ne=(0,a.useMemo)((()=>{if(r.validateMessage)return(0,y.tZ)(h.Am,{status:r.validateStatus,children:r.validateMessage})}),[r.validateMessage,r.validateStatus]),re=(0,a.useMemo)((()=>[...new Set([...n.map((e=>e[P]))])].map((e=>({label:G(e,B),value:e,isNewOption:!1})))),[n,B,P,G]),oe=(0,a.useMemo)((()=>(!J||H||(0,m.Gq)(J,re,!0)||re.unshift({label:J,value:J,isNewOption:!0}),re)),[H,J,re]),ae=(0,a.useCallback)(((e,t)=>{const n=(0,m.mj)("label");return F.sortAscending?n(e,t):n(t,e)}),[F.sortAscending]);return(0,a.useEffect)((()=>{if(D&&void 0===r.value){const e=n[0]?K.map((e=>n[0][e])):null;void 0!==(null==e?void 0:e[0])&&U(e)}else U(W?null:r.value)}),[P,W,D,k,j,U,n,K,JSON.stringify(r.value)]),(0,a.useEffect)((()=>{N(Y)}),[JSON.stringify(Y)]),(0,y.tZ)(h.un,{height:b,width:M,children:(0,y.tZ)(h.jp,{validateStatus:r.validateStatus,extra:ne,children:(0,y.tZ)(d.Ph,{allowClear:!0,allowNewOptions:!I,allowSelectAll:!I,value:r.value||[],disabled:W,getPopupContainer:Z?()=>(null==A?void 0:A.current)||document.body:e=>(null==e?void 0:e.parentNode)||document.body,showSearch:V,mode:H?"multiple":"single",placeholder:te,onClear:()=>Q(""),onSearch:Q,onBlur:X,onFocus:R,onMouseEnter:O,onMouseLeave:C,onChange:ee,ref:T,loading:w,oneLine:_===v.Bp.Horizontal,invertSelection:j,options:oe,sortComparator:ae,onDropdownVisibleChange:$})})})}},174448:(e,t,n)=>{n.d(t,{Am:()=>u,h2:()=>a,jp:()=>l,un:()=>i});var r=n(751995),o=n(804591);const a=0,i=r.iK.div`
  min-height: ${({height:e})=>e}px;
  width: ${({width:e})=>e===a?"100%":`${e}px`};
`,l=(0,r.iK)(o.Z)`
  &.ant-row.ant-form-item {
    margin: 0;
  }
`,u=r.iK.div`
  color: ${({theme:e,status:t="error"})=>{var n;return null==(n=e.colors[t])?void 0:n.base}};
`},356080:(e,t,n)=>{n.d(t,{EG:()=>u,WO:()=>l,vb:()=>i});var r=n(88889),o=n(454076),a=n(861641);const i=(e,t,n=!1,r=!1)=>{const o={};return n?o.adhoc_filters=[{expressionType:a.p.Sql,clause:a.N.Where,sqlExpression:"1 = 0"}]:null!=t&&0!==t.length&&(o.filters=[{col:e,op:r?"NOT IN":"IN",val:t}]),o},l=(e,t,n)=>{const r=[];return null!=t&&t!==n&&r.push({col:e,op:">=",val:t}),null!=n&&n!==t&&r.push({col:e,op:"<=",val:n}),null!=n&&null!=t&&n===t&&r.push({col:e,op:"==",val:n}),r.length?{filters:r}:{}};function u({timeFormatter:e,numberFormatter:t}={}){return(n,a)=>{if(null==n)return o.li;if("boolean"==typeof n)return n?o.Tb:o.jy;if(a===r.Z.Boolean)try{return JSON.parse(String(n).toLowerCase())?o.Tb:o.jy}catch{return o.jy}return"string"==typeof n?n:e&&a===r.Z.Temporal?e(n):t&&"number"==typeof n&&a===r.Z.Numeric?t(n):String(n)}}},890057:(e,t,n)=>{n.d(t,{C:()=>i,x:()=>a});var r=n(512902),o=n(667294);function a(e){var t=(0,o.useState)((function(){return(0,r.vV)("function"==typeof e?e():e,!0)})),n=t[1];return[t[0],(0,o.useCallback)((function(e){n("function"==typeof e?(0,r.Uy)(e):(0,r.vV)(e))}),[])]}function i(e,t,n){var a=(0,o.useMemo)((function(){return(0,r.Uy)(e)}),[e]);return(0,o.useReducer)(a,t,n)}}}]);
//# sourceMappingURL=ed511e915db61b94e6e9.chunk.js.map