"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7461],{827461:(e,h,l)=>{l.r(h),l.d(h,{default:()=>v});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var h=1;h<arguments.length;h++){var l=arguments[h];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const v=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M10 8h1v2h2V8h1v2h2v1h-2v2h2v1h-2v2h-1v-2h-2v2h-1v-2H8v-1h2v-2H8v-1h2zm3 5v-2h-2v2z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=186a5544d22d50ff0b57.chunk.js.map