"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[5922],{325922:(e,l,a)=>{a.r(l),a.d(l,{default:()=>s});var n,t=a(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m20.34 9.32-14-7a3 3 0 0 0-4.08 3.9l2.4 5.37c.11.262.11.558 0 .82l-2.4 5.37A3 3 0 0 0 5 22a3.14 3.14 0 0 0 1.35-.32l14-7a3 3 0 0 0 0-5.36zm-.89 3.57-14 7a1 1 0 0 1-1.35-1.3l2.39-5.37q.045-.108.08-.22h6.89a1 1 0 1 0 0-2H6.57a2 2 0 0 0-.08-.22L4.1 5.41a1 1 0 0 1 1.35-1.3l14 7a1 1 0 0 1 0 1.78",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=afb4f6ee9993ae1fd7f0.chunk.js.map