"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7539],{67539:(e,l,n)=>{n.r(l),n.d(l,{default:()=>i});var t,r,a=n(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},s.apply(null,arguments)}const i=e=>a.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M12 2C6.477 2 2 6.477 2 12s4.477 10 10 10 10-4.477 10-10A10 10 0 0 0 12 2",clipRule:"evenodd"})),r||(r=a.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"M16 11H8a1 1 0 1 0 0 2h8a1 1 0 1 0 0-2",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=ccc6a56d62a69b1a8b86.chunk.js.map