"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8960],{158960:(a,e,l)=>{l.r(e),l.d(e,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const s=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m20 8.024-6-5.26a3 3 0 0 0-4 0l-6 5.26a3 3 0 0 0-1 2.26v8.74a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-8.75a3 3 0 0 0-1-2.25m-6 12h-4v-5a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1zm5-1a1 1 0 0 1-1 1h-2v-5a3 3 0 0 0-3-3h-2a3 3 0 0 0-3 3v5H6a1 1 0 0 1-1-1v-8.75a1 1 0 0 1 .34-.75l6-5.25a1 1 0 0 1 1.32 0l6 5.25a1 1 0 0 1 .34.75z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=4bfd932355e715c55e17.chunk.js.map