"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7822],{707822:(e,l,n)=>{n.r(l),n.d(l,{default:()=>i});var t,r,a=n(667294);function c(){return c=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},c.apply(null,arguments)}const i=e=>a.createElement("svg",c({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M12 18a6 6 0 1 0 0-12 6 6 0 0 0 0 12",clipRule:"evenodd"})),r||(r=a.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"M12 16c-.552 0-1-.373-1-.833V12h-1v-1h2c.552 0 1 .373 1 .833v3.334c0 .46-.448.833-1 .833m-1-7a1 1 0 1 0 2 0 1 1 0 0 0-2 0",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=2df25150c6d4a52fb2fd.chunk.js.map