"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[1378],{451378:(e,t,h)=>{h.r(t),h.d(t,{default:()=>a});var n,r=h(667294);function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var h=arguments[t];for(var n in h)({}).hasOwnProperty.call(h,n)&&(e[n]=h[n])}return e},l.apply(null,arguments)}const a=e=>r.createElement("svg",l({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=r.createElement("path",{fill:"currentColor",fillOpacity:.85,d:"M20 10V8h-4V4h-2v4h-4V4H8v4H4v2h4v4H4v2h4v4h2v-4h4v4h2v-4h4v-2h-4v-4zm-6 4h-4v-4h4z"})))}}]);
//# sourceMappingURL=19395c965769b3c1ac5d.chunk.js.map