"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[894],{440894:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M15.945 10.79a.5.5 0 0 0-.445-.274h-2.502V7.524a.501.501 0 0 0-.906-.32L8.09 12.692a.5.5 0 0 0-.04.499.5.5 0 0 0 .445.319h2.502V16.5a.5.5 0 0 0 .5.499c.16 0 .311-.076.406-.204l4.003-5.487a.5.5 0 0 0 .04-.518m-3.949 4.18v-2.461H9.46L12 9.029l-.001 1.487v1h2.518z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=c865cebad2723503b51b.chunk.js.map