.react-checkbox-tree{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse;font-size:16px}.react-checkbox-tree>ol{-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto}.react-checkbox-tree ol{list-style-type:none;margin:0;padding-left:0}.react-checkbox-tree ol ol{padding-left:24px}.react-checkbox-tree button{color:inherit;line-height:normal}.react-checkbox-tree button:disabled{cursor:not-allowed}.react-checkbox-tree .rct-bare-label{cursor:default}.react-checkbox-tree label{cursor:pointer;margin-bottom:0}.react-checkbox-tree label:hover{background:#3333cc1a}.react-checkbox-tree label:active,.react-checkbox-tree label:focus{background:#3333cc26}.react-checkbox-tree:not(.rct-native-display) input{display:none}.react-checkbox-tree.rct-native-display input{margin:0 5px}.react-checkbox-tree .rct-icon{display:inline-block;font-family:Font Awesome\ 5 Free,FontAwesome,sans-serif;font-style:normal;font-variant:normal;font-weight:400;text-align:center;text-rendering:auto}.rct-disabled>.rct-text>label{cursor:not-allowed;opacity:.75}.rct-disabled>.rct-text>label:active,.rct-disabled>.rct-text>label:hover{background:#0000}.rct-text{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.rct-options{-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto;margin-left:.5rem;text-align:right}.rct-option{background:none;border:0;cursor:pointer;font-size:18px;opacity:.75;padding:0 4px}.rct-option:hover{opacity:1}.rct-option+.rct-option{margin-left:2px}.rct-checkbox,.rct-collapse,.rct-node-icon{padding:0 5px}.rct-checkbox *,.rct-collapse *,.rct-node-icon *{display:inline-block;margin:0;width:14px}.rct-collapse{-ms-flex-item-align:stretch;align-self:stretch;background:none;border:0;color:inherit;font-size:12px;line-height:normal}.rct-collapse.rct-collapse-btn{cursor:pointer}.rct-collapse>.rct-icon-expand-close{opacity:.5}.rct-collapse>.rct-icon-expand-close:hover{opacity:1}.rct-native-display .rct-checkbox{display:none}.rct-node-clickable{cursor:pointer}.rct-node-clickable:hover{background:#3333cc1a}.rct-node-clickable:focus{background:#33c3;outline:0}.rct-node-icon{color:#33c}.rct-title{padding:0 5px}.rct-icons-fa4 .rct-icon-expand-close:before{content:"\f054"}.rct-icons-fa4 .rct-icon-expand-open:before{content:"\f078"}.rct-icons-fa4 .rct-icon-uncheck:before{content:"\f096"}.rct-icons-fa4 .rct-icon-check:before{content:"\f046"}.rct-icons-fa4 .rct-icon-half-check:before{content:"\f046";opacity:.5}.rct-icons-fa4 .rct-icon-leaf:before{content:"\f016"}.rct-icons-fa4 .rct-icon-parent-open:before{content:"\f115"}.rct-icons-fa4 .rct-icon-parent-close:before{content:"\f114"}.rct-icons-fa4 .rct-icon-expand-all:before{content:"\f0fe"}.rct-icons-fa4 .rct-icon-collapse-all:before{content:"\f146"}.rct-icons-fa5 .rct-icon-expand-close:before{content:"\f054";font-weight:900}.rct-icons-fa5 .rct-icon-expand-open:before{content:"\f078";font-weight:900}.rct-icons-fa5 .rct-icon-uncheck:before{content:"\f0c8"}.rct-icons-fa5 .rct-icon-check:before{content:"\f14a"}.rct-icons-fa5 .rct-icon-half-check:before{content:"\f14a";opacity:.5}.rct-icons-fa5 .rct-icon-leaf:before{content:"\f15b"}.rct-icons-fa5 .rct-icon-parent-open:before{content:"\f07c"}.rct-icons-fa5 .rct-icon-parent-close:before{content:"\f07b"}.rct-icons-fa5 .rct-icon-expand-all:before{content:"\f0fe"}.rct-icons-fa5 .rct-icon-collapse-all:before{content:"\f146"}.rct-direction-rtl{direction:rtl}.rct-direction-rtl ol ol{padding-left:0;padding-right:24px}.rct-direction-rtl.rct-icons-fa4 .rct-icon-expand-close:before{content:"\f105"}.rct-direction-rtl.rct-icons-fa5 .rct-icon-expand-close:before{content:"\f053"}
/*# sourceMappingURL=DashboardContainer.41e7e14e31aa731ea8d5.chunk.css.map*/