"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[5029],{315029:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M19.25 17.535h-7.027a3.75 3.75 0 0 1-3.75-3.75v-5.25l-2.19 2.22a.75.75 0 1 1-1.065-1.057L8.69 6.225a.8.8 0 0 1 .248-.165.7.7 0 0 1 .57 0q.138.062.247.165l3.495 3.473a.75.75 0 0 1-1.057 1.057l-2.22-2.19v5.25a2.25 2.25 0 0 0 2.25 2.25h7.027a.75.75 0 0 1 0 1.5z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=0e8cc1513cab6a188eec.chunk.js.map