"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[1603],{131603:(e,l,n)=>{n.r(l),n.d(l,{default:()=>a});var t,r=n(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},s.apply(null,arguments)}const a=e=>r.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M2 12C2 6.48 6.48 2 12 2s10 4.48 10 10-4.48 10-10 10S2 17.52 2 12m18 0a8 8 0 1 1-16 0 8 8 0 0 1 16 0",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=e739748b7709ee465aba.chunk.js.map