"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8225],{548225:(e,a,l)=>{l.r(a),l.d(a,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var l=arguments[a];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M10.5 16a1 1 0 0 0 1 1h1a1 1 0 1 0 0-2h-1a1 1 0 0 0-1 1m-4-9a1 1 0 0 0 0 2h11a1 1 0 1 0 0-2zm2 5a1 1 0 0 0 1 1h5a1 1 0 1 0 0-2h-5a1 1 0 0 0-1 1",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=d94d9db5c2799e0e1ea6.chunk.js.map