"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[3613],{243613:(e,l,a)=>{a.r(l),a.d(l,{default:()=>h});var n,t=a(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},r.apply(null,arguments)}const h=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m15 8.73 5.11 8.76h-.04A3 3 0 0 1 17.48 22h-11a3 3 0 0 1-2.59-4.51L9 8.73V4H8a1 1 0 1 1 0-2h8a1 1 0 1 1 0 2h-1zM11 9a1 1 0 0 1-.14.49L10 11h4l-.86-1.5A1 1 0 0 1 13 9V4h-2zm6.52 10.99a1 1 0 0 0 .86-.5v.01a1 1 0 0 0 0-1l-3.2-5.5H8.83l-3.17 5.49a1 1 0 0 0 .86 1.5zM10 15a1 1 0 1 0 0 2 1 1 0 0 0 0-2m4 1a1 1 0 1 0 0 2 1 1 0 0 0 0-2",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=244c439564806850ea06.chunk.js.map