"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[287],{370287:(e,t,h)=>{h.r(t),h.d(t,{default:()=>s});var n,r=h(667294);function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var h=arguments[t];for(var n in h)({}).hasOwnProperty.call(h,n)&&(e[n]=h[n])}return e},l.apply(null,arguments)}const s=e=>r.createElement("svg",l({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=r.createElement("path",{fill:"currentColor",d:"M13 7.5h5v2h-5zm0 7h5v2h-5zM19 3H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2m0 16H5V5h14zM11 6H6v5h5zm-1 4H7V7h3zm1 3H6v5h5zm-1 4H7v-3h3z"})))}}]);
//# sourceMappingURL=6dd571823daca0de4e45.chunk.js.map