"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8829],{348829:(a,e,l)=>{l.r(e),l.d(e,{default:()=>h});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const h=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M12 19a1 1 0 1 0 0-2 1 1 0 0 0 0 2m5 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2m0-4a1 1 0 1 0 0-2 1 1 0 0 0 0 2m-5 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2m7-12h-1V2a1 1 0 1 0-2 0v1H8V2a1 1 0 0 0-2 0v1H5a3 3 0 0 0-3 3v14a3 3 0 0 0 3 3h14a3 3 0 0 0 3-3V6a3 3 0 0 0-3-3m1 17a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1v-9h16zm0-11H4V6a1 1 0 0 1 1-1h1v1a1 1 0 0 0 2 0V5h8v1a1 1 0 1 0 2 0V5h1a1 1 0 0 1 1 1zM7 15a1 1 0 1 0 0-2 1 1 0 0 0 0 2m0 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=59ec3f3a0fabd682f13a.chunk.js.map