"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[4749],{284749:(e,l,a)=>{a.r(l),a.d(l,{default:()=>s});var n,t=a(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M5 18.506h4.24a1 1 0 0 0 .71-.29l6.92-6.93 2.84-2.78a1 1 0 0 0 0-1.42l-4.24-4.29a1 1 0 0 0-1.42 0l-2.82 2.83-6.94 6.93a1 1 0 0 0-.29.71v4.24a1 1 0 0 0 1 1m9.76-13.59 2.83 2.83-1.42 1.42-2.83-2.83zM6 13.676l5.93-5.93 2.83 2.83-5.93 5.93H6zm15 6.83H3a1 1 0 1 0 0 2h18a1 1 0 1 0 0-2",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=cbed54efaf3d88d81302.chunk.js.map