"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7239],{477239:(a,e,l)=>{l.r(e),l.d(e,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const s=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M8.29 11.29a1 1 0 0 0 0 1.42l3 3a1 1 0 0 0 1.42 0l3-3a1.004 1.004 0 0 0-1.42-1.42L13 12.59V3a1 1 0 1 0-2 0v9.59l-1.29-1.3a1 1 0 0 0-1.42 0M18 7h-2a1 1 0 1 0 0 2h2a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1H6a1 1 0 0 1-1-1v-7a1 1 0 0 1 1-1h2a1 1 0 0 0 0-2H6a3 3 0 0 0-3 3v7a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-7a3 3 0 0 0-3-3",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=40af516a19490d0e08f3.chunk.js.map