"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7635],{347635:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M11.18 8.17a1 1 0 0 1 1.64 0l5.079 7.257a1 1 0 0 1-.82 1.573H6.921a1 1 0 0 1-.82-1.573z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=9591fabeab455857d161.chunk.js.map