"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[9684],{199684:(e,l,t)=>{t.r(l),t.d(l,{default:()=>s});var n,r,a=t(667294);function h(){return h=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},h.apply(null,arguments)}const s=e=>a.createElement("svg",h({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M21 2H3a1 1 0 0 0-1 1v18a1 1 0 0 0 1 1h18a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1M11 20H4v-4h7zm0-6H4v-4h7zm9 6h-7v-4h7zm0-6h-7v-4h7zm0-6H4V4h16z",clipRule:"evenodd"})),r||(r=a.createElement("path",{fill:"currentColor",d:"M4 4h16v4H4z"})))}}]);
//# sourceMappingURL=dfc30dd33011741b66bd.chunk.js.map