"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[6945],{216945:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"#E0E0E0",fillRule:"evenodd",d:"M9.707 9.293c-.63.63-.184 1.707.707 1.707h3.172c.89 0 1.337-1.077.707-1.707l-1.586-1.586a1 1 0 0 0-1.414 0zm0 5.414c-.63-.63-.184-1.707.707-1.707h3.172c.89 0 1.337 1.077.707 1.707l-1.586 1.586a1 1 0 0 1-1.414 0z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=ee9305f56627471b9d2c.chunk.js.map