"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[3489],{373489:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M11 12h-1v-1h2a1 1 0 0 1 1 1v4h1v1h-4v-1h1zm1.38-4.92a1 1 0 0 0-.76 0 1 1 0 0 0-.33.21 1.2 1.2 0 0 0-.21.33 1 1 0 0 0 .21 1.09q.147.133.33.21A1 1 0 0 0 13 8a1.05 1.05 0 0 0-.29-.71 1 1 0 0 0-.33-.21M12 2C6.477 2 2 6.477 2 12s4.477 10 10 10 10-4.477 10-10A10 10 0 0 0 12 2m0 18a8 8 0 1 1 0-16 8 8 0 0 1 0 16",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=443e5260d50bbb16e5b2.chunk.js.map