"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[6730],{166730:(a,e,l)=>{l.r(e),l.d(e,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const s=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M8 2h8a4 4 0 0 1 4 4v12a4 4 0 0 1-4 4H8a4 4 0 0 1-4-4V6a4 4 0 0 1 4-4m8 18a2 2 0 0 0 2-2v-2.56a3.9 3.9 0 0 1-2 .56H8a3.9 3.9 0 0 1-2-.56V18a2 2 0 0 0 2 2zm0-6a2 2 0 0 0 2-2V9.44a3.9 3.9 0 0 1-2 .56H8a3.9 3.9 0 0 1-2-.56V12a2 2 0 0 0 2 2zM8 8h8a2 2 0 1 0 0-4H8a2 2 0 1 0 0 4m0 3a1 1 0 1 0 0 2 1 1 0 0 0 0-2m0 6a1 1 0 1 0 0 2 1 1 0 0 0 0-2",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=2ce812c3a45afbdb750b.chunk.js.map