"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8082],{748082:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:18,height:18,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m10.41 9 6.3-6.29a1.004 1.004 0 1 0-1.42-1.42L9 7.59l-6.29-6.3a1.004 1.004 0 0 0-1.42 1.42L7.59 9l-6.3 6.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0L9 10.41l6.29 6.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=774a2645dff92b6c2dba.chunk.js.map