"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[971],{560971:(a,e,l)=>{l.r(e),l.d(e,{default:()=>r});var n,t=l(667294);function h(){return h=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},h.apply(null,arguments)}const r=a=>t.createElement("svg",h({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M16 14H8a1 1 0 1 0 0 2h8a1 1 0 1 0 0-2m0-4h-6a1 1 0 1 0 0 2h6a1 1 0 1 0 0-2m4-6h-3V3a1 1 0 1 0-2 0v1h-2V3a1 1 0 1 0-2 0v1H9V3a1 1 0 0 0-2 0v1H4a1 1 0 0 0-1 1v14a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3V5a1 1 0 0 0-1-1m-1 15a1 1 0 0 1-1 1H6a1 1 0 0 1-1-1V6h2v1a1 1 0 0 0 2 0V6h2v1a1 1 0 1 0 2 0V6h2v1a1 1 0 1 0 2 0V6h2z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=d97bc0282335ae5aaf5c.chunk.js.map