"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[1016],{905336:(e,h,l)=>{l.r(h),l.d(h,{default:()=>v});var t,n,a=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var h=1;h<arguments.length;h++){var l=arguments[h];for(var t in l)({}).hasOwnProperty.call(l,t)&&(e[t]=l[t])}return e},r.apply(null,arguments)}const v=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:25,fill:"none"},e),t||(t=a.createElement("path",{fill:"#136478",d:"M4 4.5h16v16H4z"})),n||(n=a.createElement("path",{fill:"#A5DBE9",fillRule:"evenodd",d:"M21 2.5H3a1 1 0 0 0-1 1v18a1 1 0 0 0 1 1h18a1 1 0 0 0 1-1v-18a1 1 0 0 0-1-1m-13 18H4v-4h4zm0-6H4v-4h4zm0-6H4v-4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4v-4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4v-4h4z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=5526788cc87876884e0f.chunk.js.map