"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[84],{100084:(e,a,l)=>{l.r(a),l.d(a,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var l=arguments[a];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M18.328 14.808h-3.624a.8.8 0 0 0 0 1.6h1.92A6.4 6.4 0 0 1 5.6 12 .8.8 0 0 0 4 12a8 8 0 0 0 13.504 5.784V19.2a.8.8 0 0 0 1.6 0v-3.6a.8.8 0 0 0-.776-.792M12 4a8 8 0 0 0-5.504 2.216V4.8a.8.8 0 1 0-1.6 0v3.6a.8.8 0 0 0 .8.8h3.6a.8.8 0 1 0 0-1.6h-1.92A6.4 6.4 0 0 1 18.4 12a.8.8 0 0 0 1.6 0 8 8 0 0 0-8-8",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=4958ab9020a4ce03151a.chunk.js.map