"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[869],{130869:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a,r=n(667294);function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},i.apply(null,arguments)}const s=e=>r.createElement("svg",i({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M19.07 4.93A10 10 0 1 0 4.93 19.07 10 10 0 1 0 19.07 4.93",clipRule:"evenodd"})),a||(a=r.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"M15.71 8.29a1 1 0 0 0-1.42 0L12 10.59l-2.29-2.3a1.004 1.004 0 0 0-1.42 1.42l2.3 2.29-2.3 2.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l2.29-2.3 2.29 2.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42L13.41 12l2.3-2.29a1 1 0 0 0 0-1.42",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=3034bbeb57cefc11d856.chunk.js.map