"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2109],{272109:(e,t,r)=>{r.r(t),r.d(t,{default:()=>a});var n,l,i=r(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},s.apply(null,arguments)}const a=e=>i.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:25,fill:"none"},e),n||(n=i.createElement("rect",{width:16,height:16,x:4,y:4.5,fill:"currentColor",rx:2})),l||(l=i.createElement("rect",{width:8,height:2,x:8,y:11.5,fill:"#fff",rx:.5})))}}]);
//# sourceMappingURL=6a4dda67c7ae3a9f1261.chunk.js.map