"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[650],{970650:(e,r,t)=>{t.r(r),t.d(r,{default:()=>a});var n,l=t(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var r=1;r<arguments.length;r++){var t=arguments[r];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},s.apply(null,arguments)}const a=e=>l.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=l.createElement("circle",{cx:12,cy:12,r:4,fill:"currentColor"})))}}]);
//# sourceMappingURL=b1764cb173fc9ee879eb.chunk.js.map