"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7850],{227850:(e,t,n)=>{n.r(t),n.d(t,{default:()=>a});var r,l=n(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},s.apply(null,arguments)}const a=e=>l.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),r||(r=l.createElement("path",{fill:"currentColor",d:"M2 2h20v20H2z"})))}}]);
//# sourceMappingURL=4fbde596c960f68f29be.chunk.js.map