"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2652],{552652:(e,t,l)=>{l.r(t),l.d(t,{default:()=>s});var n,r=l(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24},e),n||(n=r.createElement("g",{fill:"none",fillRule:"evenodd"},r.createElement("path",{d:"M0 0h24v24H0z"}),r.createElement("path",{fill:"currentColor",d:"M6.05 6.05a7 7 0 0 1 10.554 9.144l-.02.02 3.98 3.98a1 1 0 0 1 0 1.42 1 1 0 0 1-1.42 0L15.16 16.63A7 7 0 0 1 6.05 6.05m8.444 1.414a5 5 0 1 0-7.062 7.08 5 5 0 0 0 7.062-7.08"}))))}}]);
//# sourceMappingURL=ca852e96526ed0bbebe1.chunk.js.map