"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[1441],{251441:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M10 12a2 2 0 1 0 4 0 2 2 0 0 0-4 0m0 7a2 2 0 1 0 4 0 2 2 0 0 0-4 0m0-14a2 2 0 1 0 4 0 2 2 0 0 0-4 0",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=cee979e68c2fdd1f4acf.chunk.js.map