"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[5394],{965394:(e,t,l)=>{l.r(t),l.d(t,{default:()=>s});var a,n=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var a in l)({}).hasOwnProperty.call(l,a)&&(e[a]=l[a])}return e},r.apply(null,arguments)}const s=e=>n.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24},e),a||(a=n.createElement("g",{fill:"none",fillRule:"evenodd"},n.createElement("path",{d:"M0 0h24v24H0z"}),n.createElement("path",{fill:"currentColor",d:"M15.71 8.29a1 1 0 0 0-1.42 0L12 10.59l-2.29-2.3a1.004 1.004 0 0 0-1.42 1.42l2.3 2.29-2.3 2.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l2.29-2.3 2.29 2.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42L13.41 12l2.3-2.29a1 1 0 0 0 0-1.42"}))))}}]);
//# sourceMappingURL=cdadd626c144590757b0.chunk.js.map