"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[9542],{309542:(a,e,l)=>{l.r(e),l.d(e,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const s=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M18 10.82a1 1 0 0 0-1 1V19a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V8a1 1 0 0 1 1-1h7.18a1 1 0 1 0 0-2H5a3 3 0 0 0-3 3v11a3 3 0 0 0 3 3h11a3 3 0 0 0 3-3v-7.18a1 1 0 0 0-1-1m3.92-8.2a1 1 0 0 0-.54-.54A1 1 0 0 0 21 2h-6a1 1 0 1 0 0 2h3.59L8.29 14.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0L20 5.41V9a1 1 0 1 0 2 0V3a1 1 0 0 0-.08-.38",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=f4cdeb2370059664c092.chunk.js.map