"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8058],{648058:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m22.32 5.05-6-2h-.07a.7.7 0 0 0-.14 0h-.43L10 5 4.32 3.05a1 1 0 0 0-.9.14A1 1 0 0 0 3 4v14a1 1 0 0 0 .68.95l6 2a1 1 0 0 0 .62 0l5.7-1.9L21.68 21q.16.021.32 0a.94.94 0 0 0 .58-.19A1 1 0 0 0 23 20V6a1 1 0 0 0-.68-.95M9 18.61l-4-1.33V5.39l4 1.33zm6-1.33-4 1.33V6.72l4-1.33zm6 1.33-4-1.33V5.39l4 1.33z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=9f2e3a54cf2fbdc39b8d.chunk.js.map