"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8012],{888012:(e,l,t)=>{t.r(l),t.d(l,{default:()=>c});var n,r=t(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},a.apply(null,arguments)}const c=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=r.createElement("g",{fillRule:"evenodd",clipRule:"evenodd"},r.createElement("path",{fill:"#E0E0E0",d:"M10.414 13c-.89 0-1.337 1.077-.707 1.707l1.586 1.586a1 1 0 0 0 1.414 0l1.586-1.586c.63-.63.184-1.707-.707-1.707z"}),r.createElement("path",{fill:"currentColor",d:"M10.414 11c-.89 0-1.337-1.077-.707-1.707l1.586-1.586a1 1 0 0 1 1.414 0l1.586 1.586c.63.63.184 1.707-.707 1.707z"}))))}}]);
//# sourceMappingURL=9508618d61afa95caa8b.chunk.js.map