"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[511],{390511:(e,l,a)=>{a.r(l),a.d(l,{default:()=>s});var n,t=a(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M4 4v16h16V7.828l-3-3V6a2 2 0 0 1-2 2H9a2 2 0 0 1-2-2V4zm5 0h6v2H9zm6-2h2l5 5v13a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h11m-1 12a2 2 0 1 1-4 0 2 2 0 0 1 4 0m2 0a4 4 0 1 1-8 0 4 4 0 0 1 8 0",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=090e90c077ced0d04882.chunk.js.map