"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2905],{862905:(t,e,r)=>{r.r(e),r.d(e,{default:()=>h});var n,a,l,i=r(667294);function s(){return s=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)({}).hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},s.apply(null,arguments)}const h=t=>i.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},t),n||(n=i.createElement("rect",{width:8,height:2,x:10,y:8,fill:"#B2B2B2",rx:1,transform:"rotate(90 10 8)"})),a||(a=i.createElement("rect",{width:8,height:2,x:13,y:8,fill:"#666",rx:1,transform:"rotate(90 13 8)"})),l||(l=i.createElement("rect",{width:8,height:2,x:16,y:8,fill:"#323232",rx:1,transform:"rotate(90 16 8)"})))}}]);
//# sourceMappingURL=9188fb1a95ec0351b46b.chunk.js.map