"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[4255],{314255:(e,h,l)=>{l.r(h),l.d(h,{default:()=>a});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var h=1;h<arguments.length;h++){var l=arguments[h];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const a=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:25,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M21 2.5H3a1 1 0 0 0-1 1v18a1 1 0 0 0 1 1h18a1 1 0 0 0 1-1v-18a1 1 0 0 0-1-1m-13 18H4v-4h4zm0-6H4v-4h4zm0-6H4v-4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4v-4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4v-4h4z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=5eef3df4c6f9b95b23ef.chunk.js.map