"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[3702],{3702:(e,a,l)=>{l.r(a),l.d(a,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var a=1;a<arguments.length;a++){var l=arguments[a];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M19 4H5a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h14a3 3 0 0 0 3-3V7a3 3 0 0 0-3-3M5 6h14a1 1 0 0 1 1 1l-8 4.88L4 7a1 1 0 0 1 1-1m15 11a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V9.28l7.48 4.57a1 1 0 0 0 1 0L20 9.28z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=5883a30f00e68aa158b6.chunk.js.map