"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[9558],{689558:(e,l,n)=>{n.r(l),n.d(l,{default:()=>i});var t,a,r=n(667294);function s(){return s=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},s.apply(null,arguments)}const i=e=>r.createElement("svg",s({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M12 2C6.477 2 2 6.477 2 12s4.477 10 10 10 10-4.477 10-10A10 10 0 0 0 12 2",clipRule:"evenodd"})),a||(a=r.createElement("path",{fill:"#fff",fillRule:"evenodd",d:"M11.62 7.08a1 1 0 0 1 .76 0 1 1 0 0 1 .33.21c.183.192.287.445.29.71a1 1 0 0 1-1.38.92 1.2 1.2 0 0 1-.33-.21A1 1 0 0 1 11 8a.84.84 0 0 1 .08-.38q.076-.183.21-.33a1 1 0 0 1 .33-.21M12 11h-1v6h4v-1h-2v-5z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=c1b18ad13f8c09b2aa26.chunk.js.map