"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[7508],{747508:(e,l,a)=>{a.r(l),a.d(l,{default:()=>o});var n,t,r=a(667294);function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},i.apply(null,arguments)}const o=e=>r.createElement("svg",i({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m15.292 10.295-2.29 2.3v-9.59a1 1 0 1 0-2 0v9.59l-2.29-2.3a1 1 0 0 0-1.42 0 1 1 0 0 0 0 1.42l4 4q.144.137.33.21a1 1 0 0 0 .76 0 1 1 0 0 0 .33-.21l4-4a1.004 1.004 0 1 0-1.42-1.42",clipRule:"evenodd"})),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M21 12a1 1 0 0 0-1 1v6a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1v-6a1 1 0 1 0-2 0v6a3 3 0 0 0 3 3h14a3 3 0 0 0 3-3v-6a1 1 0 0 0-1-1",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=a7f9c7954ff7f3b34172.chunk.js.map