"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2663],{262663:(a,e,l)=>{l.r(e),l.d(e,{default:()=>h});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var l=arguments[e];for(var n in l)({}).hasOwnProperty.call(l,n)&&(a[n]=l[n])}return a},r.apply(null,arguments)}const h=a=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},a),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M10 18.5a1 1 0 0 0 1-1v-6a1 1 0 1 0-2 0v6a1 1 0 0 0 1 1m10-12h-4v-1a3 3 0 0 0-3-3h-2a3 3 0 0 0-3 3v1H4a1 1 0 0 0 0 2h1v11a3 3 0 0 0 3 3h8a3 3 0 0 0 3-3v-11h1a1 1 0 1 0 0-2m-10-1a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v1h-4zm7 14a1 1 0 0 1-1 1H8a1 1 0 0 1-1-1v-11h10zm-3-1a1 1 0 0 0 1-1v-6a1 1 0 1 0-2 0v6a1 1 0 0 0 1 1",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=837a34dde417f9641d52.chunk.js.map