"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[8430],{498430:(e,l,a)=>{a.r(l),a.d(l,{default:()=>i});var n,t,r=a(667294);function h(){return h=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},h.apply(null,arguments)}const i=e=>r.createElement("svg",h({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M21 20H4V3H3v18h18z",clipRule:"evenodd"})),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M17 4a1 1 0 0 0-1 1v13a1 1 0 0 0 1 1h1a1 1 0 0 0 1-1V5a1 1 0 0 0-1-1zm-6 4a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1zm-5 3a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=3cd920691b1d75e2257c.chunk.js.map