"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[4801],{734801:(e,h,l)=>{l.r(h),l.d(h,{default:()=>s});var n,t=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var h=1;h<arguments.length;h++){var l=arguments[h];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M7 7h2v2H7zm0 4h2v2H7zm4 0h2v2h-2zm6 0h-2v2h2zm-8 4H7v2h2zm4 0h-2v2h2zm2 0h2v2h-2zm-4-8h2v2h-2zm6 0h-2v2h2z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=53e43addd12f6061e244.chunk.js.map