"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[5481],{465481:(e,l,a)=>{a.r(l),a.d(l,{default:()=>s});var n,t=a(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var a=arguments[l];for(var n in a)({}).hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e},r.apply(null,arguments)}const s=e=>t.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),n||(n=t.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m7.29 11.29 4-4a1.004 1.004 0 0 1 1.42 1.42L10.41 11H19a1 1 0 1 1 0 2h-8.59l2.3 2.29a1 1 0 0 1 0 1.42 1 1 0 0 1-1.42 0l-4-4a1 1 0 0 1-.21-.33 1 1 0 0 1 0-.76 1 1 0 0 1 .21-.33M4 4a1 1 0 0 1 1 1v14a1 1 0 1 1-2 0V5a1 1 0 0 1 1-1",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=731d861684b34511704e.chunk.js.map