"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[2202],{962202:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,a=n(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"M17.535 19.25v-7.027a3.75 3.75 0 0 0-3.75-3.75h-5.25l2.22-2.19a.75.75 0 1 0-1.057-1.065L6.225 8.69a.8.8 0 0 0-.165.248.7.7 0 0 0 0 .57q.062.138.165.247l3.473 3.495a.75.75 0 0 0 1.057-1.057l-2.19-2.22h5.25a2.25 2.25 0 0 1 2.25 2.25v7.027a.75.75 0 0 0 1.5 0z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=4441dac21ae8febce590.chunk.js.map