"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[1363],{391363:(e,l,n)=>{n.r(l),n.d(l,{default:()=>s});var t,r=n(667294);function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}const s=e=>r.createElement("svg",a({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=r.createElement("path",{fill:"currentColor",fillRule:"evenodd",d:"m9.17 11.29 4.24-4.24a1 1 0 1 1 1.42 1.41L11.29 12l3.54 3.54a1 1 0 0 1 0 1.41 1 1 0 0 1-.71.29 1 1 0 0 1-.71-.29l-4.24-4.24a1 1 0 0 1 0-1.42",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=5fe0ef7c7b20dc3e66a9.chunk.js.map