"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[3348],{573348:(t,e,o)=>{o.r(e),o.d(e,{default:()=>h,getLayer:()=>c});var r=o(492219),l=o(61988),n=o(58371),a=o(363241),i=o(589691),s=o(51805),u=o(135944);function d(t){var e,o;return(0,u.BX)("div",{className:"deckgl-tooltip",children:[(0,u.tZ)(s.Z,{label:(0,l.t)("Centroid (Longitude and Latitude): "),value:`(${null==t?void 0:t.coordinate[0]}, ${null==t?void 0:t.coordinate[1]})`}),(0,u.tZ)(s.Z,{label:(0,l.t)("Threshold: "),value:`${null==t||null==(e=t.object)||null==(o=e.contour)?void 0:o.threshold}`})]})}const c=function(t,e,o,l){const i=t,{aggregation:s="SUM",js_data_mutator:u,contours:c,cellSize:h="200"}=i;let g=e.data.features;const p=null==c?void 0:c.map((t=>{const{lowerThreshold:e,upperThreshold:o,color:r,strokeWidth:l}=t;return o?{threshold:[e,o],color:[r.r,r.g,r.b]}:{threshold:e,color:[r.r,r.g,r.b],strokeWidth:l}}));return u&&(g=(0,a.Z)(i.js_data_mutator)(g)),new r.Z({id:`contourLayer-${i.slice_id}`,data:g,contours:p,cellSize:Number(h||"200"),aggregation:s.toUpperCase(),getPosition:t=>t.position,getWeight:t=>t.weight||0,...(0,n.N)(i,l,d)})},h=(0,i.G)(c,(function(t){return t.map((t=>t.position))}))}}]);
//# sourceMappingURL=93b7ffb2ada15fe2ddc4.chunk.js.map