"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[389],{280389:(e,h,l)=>{l.r(h),l.d(h,{default:()=>s});var t,n,a=l(667294);function r(){return r=Object.assign?Object.assign.bind():function(e){for(var h=1;h<arguments.length;h++){var l=arguments[h];for(var t in l)({}).hasOwnProperty.call(l,t)&&(e[t]=l[t])}return e},r.apply(null,arguments)}const s=e=>a.createElement("svg",r({xmlns:"http://www.w3.org/2000/svg",width:24,height:24,fill:"none"},e),t||(t=a.createElement("path",{fill:"currentColor",d:"M4 4h16v16H4z"})),n||(n=a.createElement("path",{fill:"#E0E0E0",fillRule:"evenodd",d:"M21 2H3a1 1 0 0 0-1 1v18a1 1 0 0 0 1 1h18a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1M8 20H4v-4h4zm0-6H4v-4h4zm0-6H4V4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4V4h4zm6 12h-4v-4h4zm0-6h-4v-4h4zm0-6h-4V4h4z",clipRule:"evenodd"})))}}]);
//# sourceMappingURL=7ce8fb673ae0cf78b823.chunk.js.map