"use strict";(globalThis.webpackChunksuperset=globalThis.webpackChunksuperset||[]).push([[349],{890349:(t,e,a)=>{a.r(e),a.d(e,{default:()=>i});var n=a(809857);function i(t){const{formData:e,queriesData:a,width:i,height:s}=t,{allColumnsX:u,entity:h,minLeafNodeEventCount:l}=e,{data:r}=a[0];if(r&&r.length>0){const t=h,e=u,a={[n.ENTITY_ID]:e=>String(e[t]),[n.EVENT_NAME]:t=>t[e],[n.TS]:t=>t.__timestamp||0===t.__timestamp?new Date(t.__timestamp):null};return{data:(0,n.cleanEvents)(r,a),height:s,initialMinEventCount:l,width:i}}return{data:null,height:s,width:i}}}}]);
//# sourceMappingURL=a443f76cb87177898281.chunk.js.map